package org.xbib.gradle.plugin.c.test

import org.gradle.testkit.runner.BuildResult
import org.gradle.testkit.runner.GradleRunner
import org.gradle.testkit.runner.TaskOutcome
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.io.TempDir

import static org.junit.jupiter.api.Assertions.assertEquals

class CPluginIntegrationTest {

    private File projectDir

    private File settingsFile

    private File buildFile

    @BeforeEach
    void setup(@TempDir File testProjectDir) throws IOException {
        this.projectDir = testProjectDir
        this.settingsFile = new File(testProjectDir, "settings.gradle")
        this.buildFile = new File(testProjectDir, "build.gradle")
    }

    @Test
    void testPlugin() {
        String settingsFileContent = '''
rootProject.name = 'c-test'
'''
        settingsFile.write(settingsFileContent)
        String buildFileContent = '''

plugins {
    id 'base'
    id 'org.xbib.gradle.plugin.c'
}

// dummy task, no source present

tasks.register('compileC', org.xbib.gradle.plugin.c.tasks.ExtendedCCompile) {
    toolChain.set(c.localTool("14.0.0", "/usr/bin/clang", ".o"))
    targetPlatform.set(c.platform())
    source(layout.projectDirectory.files("src/c").asFileTree)
    includes(layout.projectDirectory.files("src/headers"))
    compilerArgs.add("-S")
    objectFileDir.set(layout.buildDirectory.dir("out/o"))
}

'''
        buildFile.write(buildFileContent)
        BuildResult result = GradleRunner.create()
                .withProjectDir(projectDir)
                .withPluginClasspath()
                .withArguments(":compileC", "--info", "--stacktrace", "--warning-mode=all")
                .forwardOutput()
                .build()
        assertEquals(TaskOutcome.NO_SOURCE, result.task(":compileC").getOutcome())
    }
}
