package org.xbib.gradle.plugin.cmake.test

import org.gradle.testkit.runner.BuildResult
import org.gradle.testkit.runner.GradleRunner
import org.gradle.testkit.runner.TaskOutcome
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.condition.DisabledOnOs
import org.junit.jupiter.api.condition.OS
import org.junit.jupiter.api.io.TempDir

import static org.junit.jupiter.api.Assertions.assertEquals

@DisabledOnOs(OS.MAC)
class CmakePluginIntegrationTest {

    private File projectDir

    private File settingsFile

    private File buildFile

    @BeforeEach
    void setup(@TempDir File testProjectDir) throws IOException {
        this.projectDir = testProjectDir
        File f = new File(testProjectDir, 'build/src/main/cpp')
        f.mkdirs()
        File cmakeList = new File(f, 'CMakeLists.txt')
        cmakeList.write('')
        this.settingsFile = new File(testProjectDir, "settings.gradle")
        this.buildFile = new File(testProjectDir, "build.gradle")
    }

    @Test
    void testPlugin() {
        String settingsFileContent = '''
rootProject.name = 'cmake-test'
'''
        settingsFile.write(settingsFileContent)
        String buildFileContent = '''

plugins {
    id 'org.xbib.gradle.plugin.cmake'
}

'''
        buildFile.write(buildFileContent)
        BuildResult result = GradleRunner.create()
                .withProjectDir(projectDir)
                .withPluginClasspath()
                .withArguments(":cmakeConfigure", "--info", "--stacktrace", "--warning-mode=all")
                .forwardOutput()
                .build()
        assertEquals(TaskOutcome.SUCCESS, result.task(":cmakeConfigure").getOutcome())
    }
}
