package org.xbib.gradle.plugin.jdeps

import org.gradle.api.Action
import org.gradle.api.Plugin
import org.gradle.api.Project
import org.gradle.api.Task
import org.gradle.api.plugins.BasePlugin
import org.gradle.api.plugins.JavaPlugin
import org.gradle.api.plugins.JavaPluginConvention
import org.gradle.api.tasks.TaskProvider
import org.gradle.api.tasks.compile.JavaCompile

class JDepsPlugin implements Plugin<Project> {

    void apply(Project project) {
        project.plugins.apply(JavaPlugin)
        TaskProvider<JDepsReportTask> report = project.tasks.register('jdepsReport', JDepsReportTask,
            new Action<JDepsReportTask>() {
                @Override
                void execute(JDepsReportTask t) {
                    t.dependsOn(project.tasks.named('classes'))
                    t.group = BasePlugin.BUILD_GROUP
                    t.description = 'Generate a jdeps report on project classes and dependencies'
                    t.compileJava = project.tasks.named(JavaPlugin.COMPILE_JAVA_TASK_NAME, JavaCompile)
                    t.javaPluginConvention.set(project.convention.getPlugin(JavaPluginConvention))
                    t.projectName.set(project.name)
                    t.reportDir.convention(project.layout.buildDirectory.dir('reports/jdeps'))
                    t.projectConfigurations = project.configurations
                }
            })
        project.tasks.named('check').configure(new Action<Task>() {
            @Override
            void execute(Task t) {
                t.dependsOn(report)
            }
        })
    }
}
